### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import ITextWidget

# import local interfaces
from ztfy.security.schema import IPrincipal

# import Zope3 packages
from zope.interface import Attribute
from zope.schema import Object, List

# import local packages

from ztfy.security import _


class IPrincipalWidget(ITextWidget):
    """Principal widget interface"""

    principal = Object(title=_("Principal"),
                       schema=IPrincipal)


class IPrincipalListWidget(ITextWidget):
    """Principal list widget interface"""

    principals = List(title=_("Principals"),
                      value_type=Object(schema=IPrincipal))

    principals_map = Attribute(_("Principals map"))



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from z3c.form import button, field
from z3c.formjs import jsaction
from zope.interface import Interface

# import local packages
from ztfy.skin.form import EditForm

from ztfy.security import _


class IEditFormButtons(Interface):
    """Default edit form buttons"""

    submit = button.Button(title=_("Submit"))
    reset = jsaction.JSButton(title=_("Reset"))


class RolesEditForm(EditForm):
    """Roles edit form"""

    legend = _("Define roles on current context")

    interfaces = ()
    buttons = button.Buttons(IEditFormButtons)

    @property
    def fields(self):
        return field.Fields(*self.interfaces)

    @button.handler(buttons['submit'])
    def submit(self, action):
        super(RolesEditForm, self).handleApply(self, action)

    @jsaction.handler(buttons['reset'])
    def reset(self, event, selector):
        return 'document.location.reload();'



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.schema.interfaces import IField
from zope.security.interfaces import IPrincipal

# import local interfaces
from ztfy.security.interfaces import ISecurityManager

# import Zope3 packages

# import local packages

from ztfy.security import _


class RolePrincipalsProperty(object):
    """Principals list property matching local role owners"""

    def __init__(self, field, role, name=None, output=str, **args):
        if not IField.providedBy(field):
            raise ValueError, _("Provided field must implement IField interface...")
        if output not in (str, list):
            raise ValueError, _("Field output must be 'list' or 'str' types")
        if name is None:
            name = field.__name__
        self.__field = field
        self.__name = name
        self.__role = role
        self.__output = output
        self.__args = args

    def __get__(self, instance, klass):
        if instance is None:
            return self
        sm = ISecurityManager(instance, None)
        if sm is None:
            result = []
        else:
            result = sm.getLocalAllowedPrincipals(self.__role)
        if self.__output is str:
            return ','.join(result)
        else:
            return result

    def __set__(self, instance, value):
        if value is None:
            value = []
        elif isinstance(value, (str, unicode)):
            value = value.split(',')
        for i, v in enumerate(value):
            if IPrincipal.providedBy(v):
                value[i] = v.id
        field = self.__field.bind(instance)
        field.validate(value)
        if field.readonly:
            raise ValueError(self.__name, _("Field is readonly"))
        sm = ISecurityManager(instance, None)
        principals = sm.getLocalAllowedPrincipals(self.__role)
        removed = set(principals) - set(value)
        added = set(value) - set(principals)
        for principal in removed:
            sm.unsetRole(self.__role, principal)
        for principal in added:
            sm.grantRole(self.__role, principal)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages
import transaction

# import Zope3 interfaces
from zope.catalog.interfaces import ICatalog
from zope.component.interfaces import IComponentRegistry, ISite
from zope.intid.interfaces import IIntIds
from zope.processlifetime import IDatabaseOpenedWithRoot

# import local interfaces
from ztfy.security.interfaces import ILocalRoleIndexer
from ztfy.utils.interfaces import INewSiteManagerEvent

# import Zope3 packages
from zc.catalog.catalogindex import SetIndex
from zope.app.publication.zopepublication import ZopePublication
from zope.catalog.catalog import Catalog
from zope.component import adapter, queryUtility
from zope.intid import IntIds
from zope.location import locate
from zope.site import hooks

# import local packages
from ztfy.security.indexer import ALL_ROLES_INDEX_NAME
from ztfy.utils.site import locateAndRegister


def updateDatabaseIfNeeded(context):
    """Check for missing utilities at application startup"""
    try:
        sm = context.getSiteManager()
    except:
        return
    default = sm['default']
    # Check for required IIntIds utility
    intids = queryUtility(IIntIds)
    if intids is None:
        intids = default.get('IntIds')
        if intids is None:
            intids = IntIds()
            locate(intids, default)
            IComponentRegistry(sm).registerUtility(intids, IIntIds, '')
            default['IntIds'] = intids
    # Check for security catalog and indexes
    catalog = default.get('SecurityCatalog')
    if catalog is None:
        catalog = Catalog()
        locateAndRegister(catalog, default, 'SecurityCatalog', intids)
        IComponentRegistry(sm).registerUtility(catalog, ICatalog, 'SecurityCatalog')
    if catalog is not None:
        if ALL_ROLES_INDEX_NAME not in catalog:
            index = SetIndex(ALL_ROLES_INDEX_NAME, ILocalRoleIndexer, False)
            locateAndRegister(index, catalog, ALL_ROLES_INDEX_NAME, intids)


@adapter(IDatabaseOpenedWithRoot)
def handleOpenedDatabase(event):
    db = event.database
    connection = db.open()
    root = connection.root()
    root_folder = root.get(ZopePublication.root_name, None)
    for site in root_folder.values():
        if ISite(site, None) is not None:
            hooks.setSite(site)
            updateDatabaseIfNeeded(site)
            transaction.commit()


@adapter(INewSiteManagerEvent)
def handleNewSiteManager(event):
    updateDatabaseIfNeeded(event.object)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import IFieldWidget
from zope.schema.interfaces import IField

# import local interfaces
from ztfy.security.browser.widget.interfaces import IPrincipalWidget, IPrincipalListWidget
from ztfy.skin.layer import IZTFYBrowserLayer

# import Zope3 packages
from z3c.form.browser.text import TextWidget
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.interface import implementer, implementsOnly

# import local packages
from ztfy.jqueryui import jquery_multiselect
from ztfy.security.browser import ztfy_security
from ztfy.security.search import getPrincipal


class PrincipalWidget(TextWidget):
    """Principal widget"""

    implementsOnly(IPrincipalWidget)

    @property
    def principal(self):
        return getPrincipal(self.value)

    def render(self):
        ztfy_security.need()
        jquery_multiselect.need()
        return super(PrincipalWidget, self).render()


@adapter(IField, IZTFYBrowserLayer)
@implementer(IFieldWidget)
def PrincipalFieldWidget(field, request):
    """IPrincipalWidget factory for Principal fields"""
    return FieldWidget(field, PrincipalWidget(request))


class PrincipalListWidget(TextWidget):
    """Principals list widget"""

    implementsOnly(IPrincipalListWidget)

    @property
    def principals(self):
        if not hasattr(self, '_v_principals'):
            self._v_principals = sorted([getPrincipal(v) for v in self.value.split(',')], key=lambda x: x.title)
        return self._v_principals

    @property
    def principals_map(self):
        return '{ %s }' % ',\n'.join(["'%s': '%s'" % (principal.id, principal.title.replace("'", "&#039;"))
                                      for principal in self.principals])

    def render(self):
        ztfy_security.need()
        jquery_multiselect.need()
        return super(PrincipalListWidget, self).render()


@adapter(IField, IZTFYBrowserLayer)
@implementer(IFieldWidget)
def PrincipalListFieldWidget(field, request):
    """IPrincipalListWidget factory for PrincipalList fields"""
    return FieldWidget(field, PrincipalListWidget(request))



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.authentication.interfaces import IAuthentication, PrincipalLookupError
from zope.pluggableauth.interfaces import IPrincipalInfo
from zope.pluggableauth.plugins.groupfolder import IGroupFolder
from zope.pluggableauth.plugins.principalfolder import IInternalPrincipalContainer

# import local interfaces
from ztfy.security.interfaces import IAuthenticatorSearchAdapter

# import Zope3 packages
from zope.component import adapts, queryUtility
from zope.i18n import translate
from zope.interface import implements

# import local packages
from ztfy.utils.request import getRequest

from ztfy.security import _


class NoPrincipal(object):

    implements(IPrincipalInfo)

    def __init__(self):
        self.id = ''

    title = _("No selected principal")
    description = _("No principal was selected")


class MissingPrincipal(object):

    implements(IPrincipalInfo)

    def __init__(self, uid):
        self.id = uid
        self.request = getRequest()

    @property
    def title(self):
        return translate(_("< missing principal %s >"), context=self.request) % self.id

    @property
    def description(self):
        return translate(_("This principal can't be found in any authentication utility..."), context=self.request)


class PrincipalFolderSearchAdapter(object):
    """Principal folder search adapter"""

    adapts(IInternalPrincipalContainer)
    implements(IAuthenticatorSearchAdapter)

    def __init__(self, context):
        self.context = context

    def search(self, query):
        return self.context.search({ 'search': query })


class GroupFolderSearchAdapter(object):
    """Principal group search adapter"""

    adapts(IGroupFolder)
    implements(IAuthenticatorSearchAdapter)

    def __init__(self, context):
        self.context = context

    def search(self, query):
        return self.context.search({ 'search': query })


def getPrincipal(uid, auth=None):
    """Get a principal"""
    if not uid:
        return NoPrincipal()
    if auth is None:
        auth = queryUtility(IAuthentication)
    if auth is None:
        return NoPrincipal()
    try:
        return auth.getPrincipal(uid)
    except PrincipalLookupError:
        return MissingPrincipal(uid)


def findPrincipals(query):
    """Search for principals"""
    query = query.strip()
    if not query:
        return ()
    auth = queryUtility(IAuthentication)
    if auth is None:
        return ()
    result = []
    for _name, plugin in auth.getQueriables():
        search = IAuthenticatorSearchAdapter(plugin.authplugin, None)
        if search is not None:
            result.extend([getPrincipal(uid, auth) for uid in search.search(query)])
    return sorted(result, key=lambda x: x.title)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.tales.interfaces import ITALESFunctionNamespace

# import local interfaces
from ztfy.security.tal.interfaces import ISecurityPermissionTalesAPI

# import Zope3 packages
from zope.interface import implements

# import local packages
from ztfy.security.security import getSecurityManager


class SecurityPermissionTalesAdapter(object):
    """Security permission TALES adapter"""

    implements(ISecurityPermissionTalesAPI, ITALESFunctionNamespace)

    def __init__(self, context):
        self.context = context
        self.sm = getSecurityManager(context)

    def setEngine(self, engine):
        self.request = engine.vars['request']

    def __getattr__(self, permission):
        if self.sm is None:
            return False
        return self.sm.canUsePermission(permission)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces
from zope.annotation.interfaces import IAnnotatable

# import local interfaces
from ztfy.security.interfaces import ILocalRoleManager, ISecurityManager, ILocalRoleIndexer

# import Zope3 packages
from zope.component import adapts
from zope.interface import implements
from zope.security.checker import CheckerPublic
from zope.security.proxy import removeSecurityProxy

# import local packages


ALL_ROLES_INDEX_NAME = 'all_roles'


class LocalRoleIndexer(object):
    """Local role indexer helper interface"""

    adapts(IAnnotatable)
    implements(ILocalRoleIndexer)

    def __init__(self, context):
        self.context = context

    __Security_checker__ = CheckerPublic

    def __getattr__(self, name):
        result = set()
        sm = ISecurityManager(self.context, None)
        if sm is None:
            return result
        if name == ALL_ROLES_INDEX_NAME:
            manager = ILocalRoleManager(self.context, None)
            if manager is None:
                return result
            names = manager.__roles__
        else:
            names = (name,)
        for name in names:
            result |= removeSecurityProxy(sm.getLocalAllowedPrincipals(name))
        return result
